/* Define the working directory */
%let loc = C:\Users\sasdemo\RD_Examples ;
libname rdexamp "&loc";

/* Create sample instrument data*/
data RDexamp.Inst_Data;
    informat Mat_Date date7.;
    format Mat_Date date7.;
    length Currency $3.;
    input InstType $ InstID $ Par_LC Coup_Freq Coupon
        Mat_Date Company $ Currency $;
    datalines;
CoupBond bond1 100000 6 0.05 01JAN03 EuroCorp EUR
CoupBond bond2 100000 6 0.05 01JAN04 BritCorp GBP
CoupBond bond3 100000 6 0.05 01JAN05 AmerCorp USD
;


/* Create sample market data*/
data RDExamp.Mkt_Data;
    input EUR_6M EUR_12M GBP_6M GBP_12M USD_6M USD_12M
        USD_36M USD_EUR USD_GBP;
    datalines;
0.0303 0.0301
0.0399 0.0410
0.0149 0.0130 0.0232
1.0013 1.5711
;

proc risk;
    /* Create a new environment */
    environment new = RDExamp.Example3;

    /* Save the environment */
    environment save;
run;

proc risk;
    environment open = RDExamp.Example3;
/* Declare Instrument Variables */
    declare instvars = (
        Par_LC num var label = "Par Value (Local Currency)",
        Coup_Freq num var label = "Coupon Frequency",
        Coupon num var label = "Coupon Rate",
        Mat_Date date var label = "Maturation Date",
        Company char 15 class label = "Issuing Company" ); 
/* Declare Risk Factor Variables */
    declare riskfactors = (
        USD_EUR num FX_Spot
        fromcur = EUR
        tocur = USD
        group = "Risk Factor"
        label = "USD/EUR Exchange Rate",

        USD_GBP num FX_Spot
        fromcur = GBP
        tocur = USD
        group = "Risk Factor"
        label = "USD/GBP Exchange Rate",

        EUR_6M num IR
        currency = EUR
        maturity = 6 month
        group = "Risk Factor"
        label = "6-month interest rate (EUR)",

        EUR_12M num IR
        currency = EUR
        maturity = 12 month
        group = "Risk Factor"
        label = "12-month interest rate (EUR)",

        GBP_6M num IR
        currency = GBP
        maturity = 6 month
        group = "Risk Factor"
        label = "6-month interest rate (GBP)",


 
        GBP_12M num IR
        currency = GBP
        maturity = 12 month
        group = "Risk Factor"
        label = "12-month interest rate (GBP)",

        USD_6M num IR
        currency = USD
        maturity = 6 month
        group = "Risk Factor"
        label = "6-month interest rate (USD)",

        USD_12M num IR
        currency = USD
        maturity = 12 month
        group = "Risk Factor"
        label = "12-month interest rate (USD)",

        USD_36M num IR
        currency = USD
        maturity = 36 month
        group = "Risk Factor"
        label = "36-month interest rate (USD)" ); 
/* Declare a Reference Variable */
    declare reference = (
        loccur array IR
        group = "Risk Factor"
        label = "Int. Rate Curve in Local Currency" );
/* Define Risk Factor Curves */
    array
        EUR_Rates IR currency = EUR
        elements = (EUR_6M EUR_12M)
        group = "Risk Factor"
        label = "Euro Interest Rates"
        refmap = (loccur = "EUR"),

        GBP_Rates IR currency = GBP
        elements = (GBP_6M GBP_12M)
        group = "Risk Factor"
        label = "GBP Interest Rates"
        refmap = (loccur = "GBP"),

        USD_Rates IR currency = USD
        elements = (USD_6M USD_12M USD_36M)
        group = "Risk Factor"
        label = "USD Interest Rates"
        refmap = (loccur = "USD"); 
/* Create a Cross-Classification */
    crossclass cc1 (Currency);
environment save;
run;

/* Create a Pricing Method Program */ 
      proc compile outlib = RDExamp.Example3 
               environment = RDExamp.Example3  
               package = Coup_Bond; 
   
   /*- Pricing Function for Government Bonds -*/ 
       function govbondprc( valdate, parvalue, freq, 
                          coupon, matdate, spotval[*],  
                          maturity[*] )  
                          kind = pricing  
                          label = "Coupon Bond Pricing"; 
   
        array date{50} date1-date50; 
        npoint = dim(spotval); 
   
   /*- case of stale -*/ 
       if valdate > matdate then return(0); 
   
   /*- split parts -*/ 
       valday = day(valdate); 
       valmonth = month(valdate); 
       valyear = year(valdate);  
   
       matday = day(matdate); 
       matmonth = month(matdate); 
       matyear = year(matdate); 
   
   /*- calculate no. of flows -*/ 
       mondiff = (matyear - valyear) * 12 +  
                  matmonth - valmonth; 
       moninc = mod(mondiff, freq); 
       
       if moninc = 0 or (valday > matday) 
        then moninc = freq;  
        nflow = (mondiff-moninc)/freq + 1; 
   
   /*- calculate first date -*/  
       nextday = matday;  
       nextmon = mod(valmonth + moninc, 12); 
   
       if nextmon = 0 
          then nextmon = 12; 
   
       nextyear = valyear +  
                 ( valmonth + moninc - nextmon )/12; 
       if nextmon eq 2 and nextday > 28 then do; 
          if ( mod( nextyear, 4) eq 0 ) 
               then nextday = 29; 
          else nextday = 28; 
       end; 
   
       if (nextmon eq 4 or nextmon eq 6 or  
           nextmon eq 9 or nextmon eq 11)  
       and nextday > 30 then nextday = 30; 
   
       nextnum = nextday * 1000000 + 
       nextmon * 10000 + nextyear;  
       nextchar = put( nextnum, 8.); 
       nextdate = input( nextchar, ddmmyy8.);  
   
       date{1} = nextdate;


/*- calculate rest of the dates -*/  
       do i = 2 to nflow; 
           lastmon = month(date{i-1});  
           lastyear = year(date{i-1});  
   
           nextday = matday;  
           nextmon = mod(lastmon + freq, 12); 
   
        if nextmon = 0 
           then nextmon = 12; 
   
        nextyear = lastyear +  
                 ( lastmon + freq - nextmon )/12;  
        if nextmon eq 2 and nextday > 28 then do;  
           if ( mod( nextyear, 4) eq 0 )  
              then nextday = 29;  
           else nextday = 28;  
         end; 
        if (nextmon eq 4 or nextmon eq 6 or  
            nextmon eq 9 or nextmon eq 11)  
        and nextday > 30 then nextday = 30; 
   
            nextnum = nextday * 1000000 +  
                      nextmon * 10000 + nextyear; 
            nextchar = put( nextnum, 8.);  
            nextdate = input( nextchar, ddmmyy8.); 
   
            date{i} = nextdate;  
       end; 
   
   /*- calculate periodicity in daycounts -*/ 
       q = freq * 30/360; 
   
   /*- price -*/ 
       prc = 0; 
       do i = 1 to nflow; 
          time = intck( "day", valdate, date{i})/365;


/*- get spotrates by simple linear interpolation -*/  
          if ( time <= maturity{1} ) 
              then spotrate = spotval{1};  
          if ( time >= maturity{npoint} ) 
              then spotrate = spotval{npoint}; 
   
          if (time > maturity{1}) and 
             (time < maturity{npoint}) then do;  
             do j = 1 to npoint-1; 
               if (time > maturity{j}) and 
                  (time <= maturity{j+1}) then do;  
                   spotrate = spotval{j} +  
                   ( ( spotval{j + 1} - spotval{j} )  
                   * ( time - maturity{j} )  
                   / ( maturity{j+1} - maturity{j} ) );  
                   go to prc;  
                end;  
             end; 
          end;  
     /*- price coupons -*/  
        prc: prc = prc + parvalue * coupon * q * 
                       exp ( - (spotrate * time) );  
   
        /*- price parvalue -*/  
        if (i eq nflow) then do;  
            prc = prc + parvalue * 
                  exp ( - (spotrate * time) ); 
           end;  
        end; 
        return( prc ); 
     endsub; 


 
/* Create the Instrument Pricing Method */ 
        method Coup_Bond_Prc desc = "Coup Bond Prc"  
                          kind = Price; 
           _VALUE_ = GOVBONDPRC(_date_, Par_LC, Coup_Freq, Coupon,  
                             Mat_Date, loccur.currency,  
                             loccur.currency.mat);  
        endmethod; 

	   /* Use PV_Cashflow_Function to compute CF_Bond_Prices */
	      method CF_Bond_Prc desc = "cashflow Bond Pricing"  
                          kind = Price; 
        _VALUE_ = pv_cashflow(.,.,.,"loccur",currency);  
        endmethod;

     run; 

proc risk; 
     environment open = RDExamp.Example3; 
   
     /* Create an Instrument Template */ 
     instrument Bond_Root  
                label = "Bond Common Variables" 
                variables = (currency,  
                             holding) 
                defaults = (holding 1); 
   
     /* Create a Coupon Bond Instrument */ 
     instrument CoupBond  
                label = "Coupon Bond" 
                basetype = Bond_Root 
                variables = (Par_LC,  
                             Coup_Freq,  
                             Coupon,  
                             Mat_Date) 
                methods = (price Coup_Bond_Prc); 
   
     /* Register the Instrument Data */ 
     instdata Coup_Bond_Data 
        file = RDExamp.Inst_Data
        format = simple 
        variables = ( Par_LC Coup_Freq Coupon  
                      Mat_Date Company ); 
   
     environment save; 
   
  run; 

proc risk; 
     environment open = RDExamp.Example3; 
   
     /* Create Portfolio List */ 
     sources Coup_List Coup_Bond_Data; 
   
     /* Create Portfolio File */ 
     read sources = Coup_List out = Coup_Bond_File; 
   
     environment save; 
   
  run; 

proc risk; 
     environment open = RDExamp.Example3; 
     /* Register Market Data */ 
     marketdata Mkt_Data 
                file = RDExamp.Mkt_Data
                type = current; 
   
     environment save; 
run; 
proc risk; 
     environment open = RDExamp.Example3; 
      
     /* Profit/Loss Curve Analysis */ 
     plcurve Bond_Curve 
             curves = (USD_6M  min = 0.005 max = 0.025 n = 21 value,       
                       USD_12M min = 0.005 max = 0.025 n = 21 value,  
                       USD_36M min = 0.015 max = 0.035 n = 21 value); 
      
     /* Profit/Loss Surface Analysis */ 
     plsurface Bond_Surface 
               USD_GBP (min = 1 max = 2 n = 21 value)* 
               GBP_12M (min = 0.01 max = 0.07 n = 21 value);  
   
     /* Sensitivity Analysis */ 
     sensitivity Bond_Sensitivity 
                 vars = (USD_GBP GBP_6M GBP_12M  
                         USD_6M USD_12M USD_36M ) hessian;
/* Create an analysis project */ 
     project Coup_Bonds  
             data = (Mkt_Data)  
             portfolio = Coup_Bond_File 
             analysis = (Bond_Curve Bond_Surface  
                         Bond_Sensitivity )  
             numeraire = USD 
             crossclass = cc1 
             rundate = "01NOV02"d 
             out = Bond_Results; 
   
     /* Run the analysis project */ 
     runproject Coup_Bonds date = "01NOV02"d; 
     environment save; 
  run;

 /* Create the historical interest rate and exchange rate data */
data RDExamp.Hist_Data;
    informat Date date7.;
    format Date date7.;
    input Date EUR_6M EUR_12M GBP_6M GBP_12M USD_6M
    	USD_12M USD_36M USD_EUR USD_GBP;
   


 datalines;
01Jan99 0.0309 0.0306 0.0567 0.0549 0.0451
0.0449 0.0461 1.1591 1.6498
01Feb99 0.0304 0.0303 0.0540 0.0533 0.0470
0.0461 0.0490 1.1203 1.6276
01Mar99 0.0303 0.0305 0.0535 0.0535 0.0478
0.0465 0.0511 1.0886 1.6213
01Apr99 0.0270 0.0276 0.0530 0.0532 0.0469
0.0454 0.0503 1.0701 1.6089
01May99 0.0260 0.0268 0.0534 0.0543 0.0485
0.0475 0.0533 1.0630 1.6154
01Jun99 0.0268 0.0284 0.0524 0.0545 0.0510
0.0503 0.0570 1.0377 1.5950
01Jul99 0.0290 0.0303 0.0535 0.0554 0.0503
0.0475 0.0562 1.0370 1.5751
01Aug99 0.0305 0.0324 0.0560 0.0589 0.0520
0.0509 0.0577 1.0605 1.6058
01Sep99 0.0311 0.0330 0.0586 0.0620 0.0525
0.0508 0.0575 1.0497 1.6247
01Oct99 0.0346 0.0369 0.0614 0.0648 0.0543
0.0520 0.0594 1.0706 1.6572
01Nov99 0.0348 0.0369 0.0603 0.0639 0.0555
0.0543 0.0592 1.0328 1.6205
01Dec99 0.0351 0.0382 0.0622 0.0661 0.0584
0.0568 0.0614 1.0110 1.6132
01Jan00 0.0356 0.0395 0.0640 0.0685 0.0612
0.0576 0.0649 1.0131 1.6404
01Feb00 0.0373 0.0411 0.0642 0.0679 0.0622
0.0600 0.0665 0.9834 1.6000
01Mar00 0.0394 0.0427 0.0642 0.0674 0.0622
0.0611 0.0653 0.9643 1.5799
01Apr00 0.0408 0.0437 0.0647 0.0676 0.0615
0.0607 0.0636 0.9449 1.5823
01May00 0.0454 0.0485 0.0646 0.0673 0.0633
0.0639 0.0677 0.9059 1.5090
01Jun00 0.0468 0.0498 0.0636 0.0659 0.0617
0.0624 0.0643 0.9505 1.5092
01Jul00 0.0483 0.0510 0.0631 0.0648 0.0608
0.0627 0.0628 0.9386 1.5076
01Aug00 0.0501 0.0525 0.0633 0.0651 0.0618
0.0635 0.0617 0.9045 1.4889
01Sep00 0.0503 0.0522 0.0631 0.0645 0.0613
0.0625 0.0602 0.8695 1.4336
01Oct00 0.0510 0.0522 0.0624 0.0635 0.0601
0.0632 0.0585 0.8525 1.4506
01Nov00 0.0513 0.0519 0.0611 0.0616 0.0609
0.0634 0.0579 0.8552 1.4258
01Dec00 0.0492 0.0488 0.0594 0.0591 0.0560
0.0592 0.0526 0.8983 1.4629
01Jan01 0.0467 0.0457 0.0574 0.0566 0.0481
0.0515 0.0477 0.9376 1.4775
01Feb01 0.0467 0.0459 0.0566 0.0559 0.0468
0.0489 0.0471 0.9205 1.4525
01Mar01 0.0457 0.0447 0.0545 0.0538 0.0430
0.0444 0.0443 0.9083 1.4445
01Apr01 0.0457 0.0448 0.0530 0.0524 0.0398
0.0399 0.0442 0.8925 1.4348
01May01 0.0456 0.0453 0.0522 0.0527 0.0378
0.0374 0.0451 0.8753 1.4265
01Jun01 0.0435 0.0431 0.0533 0.0553 0.0358
0.0356 0.0435 0.8530 1.4020
01Jul01 0.0439 0.0431 0.0533 0.0556 0.0362
0.0356 0.0431 0.8615 1.4148
01Aug01 0.0422 0.0411 0.0501 0.0512 0.0347
0.0339 0.0404 0.9014 1.4372
01Sep01 0.0387 0.0377 0.0468 0.0473 0.0282
0.0271 0.0345 0.9114 1.4638
01Oct01 0.0346 0.0337 0.0439 0.0446 0.0233
0.0217 0.0314 0.9050 1.4501
01Nov01 0.0326 0.0320 0.0400 0.0411 0.0218
0.0192 0.0322 0.8883 1.4356
01Dec01 0.0325 0.0330 0.0407 0.0435 0.0222
0.0182 0.0362 0.8912 1.4413
01Jan02 0.0334 0.0348 0.0414 0.0452 0.0216
0.0177 0.0356 0.8832 1.4322
01Feb02 0.0340 0.0359 0.0419 0.0457 0.0223
0.0186 0.0355 0.8707 1.4227
01Mar02 0.0350 0.0382 0.0433 0.0478 0.0257
0.0206 0.0414 0.8766 1.4230
01Apr02 0.0353 0.0386 0.0437 0.0479 0.0248
0.0198 0.0401 0.8860 1.4429
01May02 0.0362 0.0395 0.0437 0.0481 0.0235
0.0191 0.0380 0.9170 1.4598
01Jun02 0.0359 0.0387 0.0438 0.0478 0.0220
0.0183 0.0349 0.9561 1.4837
01Jul02 0.0348 0.0364 0.0419 0.0448 0.0196
0.0174 0.0301 0.9935 1.5565
01Aug02 0.0338 0.0344 0.0401 0.0414 0.0176
0.0164 0.0252 0.9781 1.5368
01Sep02 0.0327 0.0324 0.0400 0.0407 0.0172
0.0164 0.0232 0.9806 1.5563
01Oct02 0.0317 0.0312 0.0397 0.0404 0.0165
0.0159 0.0225 0.9812 1.5575
01Nov02 0.0303 0.0301 0.0399 0.0410 0.0149
0.0130 0.0232 1.0013 1.5711
;

proc model data = RDExamp.Hist_Data;
    id Date;
    endo USD_6M USD_12M USD_36M GBP_6M GBP_12M EUR_6M
        EUR_12M USD_EUR USD_GBP;
    parms a1-a9;
    eq.one = (EUR_6M - lag(EUR_6M)) /lag(EUR_6M) - (a1);
    eq.two = (EUR_12M - lag(EUR_12M))/lag(EUR_12M) - (a2);
    eq.three = (GBP_6M - lag(GBP_6M)) /lag(GBP_6M) - (a3);
    eq.four = (GBP_12M - lag(GBP_12M))/lag(GBP_12M) - (a4);
    eq.five = (USD_6M - lag(USD_6M)) /lag(USD_6M) - (a5);
    eq.six = (USD_12M - lag(USD_12M))/lag(USD_12M) - (a6);
    eq.seven = (USD_36M - lag(USD_36M))/lag(USD_36M) - (a7);
    eq.eight = (USD_EUR - lag(USD_EUR))/lag(USD_EUR) - (a8);
    eq.nine = (USD_GBP - lag(USD_GBP))/lag(USD_GBP) - (a9);
    label a1="EUR_6M Mean" a2="EUR_12M Mean"
        a3="GBP_6M Mean" a4="GBP_12M Mean"
        a5="USD_6M Mean" a6="USD_12M Mean"
        a7="USD_36M Mean" a8="USD_EUR Mean"
        a9="USD_GBP Mean";
    fit one two three four five six seven eight nine /
        outcat = (RDExamp.example3 interval = month
        modname = Mean_Model
        modlabel = "Mean Model");
quit; 
/* Create a "difference" model for use in Monte Carlo analysis */
proc model data = RDExamp.Hist_Data;
    id Date;
    endo USD_6M USD_12M USD_36M GBP_6M GBP_12M EUR_6M
        EUR_12M USD_EUR USD_GBP;
    parms a1-a7 b0-b3 g0-g3;
    eq.a = (EUR_6M - lag(EUR_6M)) /lag(EUR_6M) - (a1);
    eq.b = (EUR_12M - lag(EUR_12M))/lag(EUR_12M) - (a2);
    eq.c = (GBP_6M - lag(GBP_6M)) /lag(GBP_6M) - (a3);
    eq.d = (GBP_12M - lag(GBP_12M))/lag(GBP_12M) - (a4);
    eq.e = (USD_6M - lag(USD_6M)) /lag(USD_6M) - (a5);
    eq.f = (USD_12M - lag(USD_12M))/lag(USD_12M) - (a6);
    eq.g = (USD_36M - lag(USD_36M))/lag(USD_36M) - (a7);
    eq.h = USD_EUR - (b0 + b1*(USD_6M-EUR_6M) +
    b2*(USD_12M-EUR_12M) + b3*USD_36M);
    eq.j = USD_GBP - (g0 + g1*(USD_6M-GBP_6M) +
    g2*(USD_12M-GBP_12M) + g3*USD_36M);
    label a1="EUR_6M Mean" a2="EUR_12M Mean"
        a3="GBP_6M Mean" a4="GBP_12M Mean"
        a5="USD_6M Mean" a6="USD_12M Mean"
        a7="USD_36M Mean"
        b0="Constant (EUR)" b1="6M Diff (EUR)"
        b2="12M Diff (EUR)" b3="36M Level (EUR)"
        g0="Constant (GBP)" g1="6M Diff (GBP)"
        g2="12M Diff (GBP)" g3="36M Level (GBP)";
    fit a b c d e f g h j /
        outcat = (RDExamp.example3 interval = month
        modname = Diff_Model
        modlabel = "Difference Model");
quit;
proc risk;
    environment open = RDexamp.example3;
    /* Register the historical data matrix */
    marketdata Hist_Data
        file = RDexamp.Hist_Data
        type = timeseries
        interval = month;
        environment save;
run; 

proc risk;
    environment open = RDexamp.example3;
    /* Create a Delta-Normal analysis */
    deltanormal Delta
        data = Hist_Data
        interval = month;
/* Create a Monte Carlo analysis */
    simulation MonteCarlo
        interval = month
        method = montecarlo
        seed = 12345
        ndraw = 1500
        errormodel = asfit
        horizon = (1);  
/* Create an Analysis Project for the */
    /* Delta Normal analysis */
    project Delta
        data = (Mkt_Data)
        portfolio = Coup_Bond_File
        analysis = (Delta)
        numeraire = USD
        crossclass = cc1
        rundate = "01NOV02"d
        out = Delta;

    /* Create an Analysis Project for the */
    /* mean model Monte Carlo */
    project MeanMC
        data = (Mkt_Data Hist_Data)
        portfolio = Coup_Bond_File
        analysis = (MonteCarlo)
        model = (RDexamp.example3.Mean_Model)
        crossclass = cc1
        numeraire = USD
        rundate = "01NOV02"d
        out = MeanMC;

    /* Create an Analysis Project for the */
    /* difference model Monte Carlo */
    project DiffMC
        data = (Mkt_Data Hist_Data)
        portfolio = Coup_Bond_File
        analysis = (MonteCarlo)
        model = (RDexamp.example3.Diff_Model)
        crossclass = cc1
        numeraire = USD
        rundate = "01NOV02"d
        out = DiffMC; 
/* Run an Analysis Project */
    runproject Delta options = (alpha = 0.05);
    runproject MeanMC options = (alpha = 0.05);
    runproject DiffMC options = (alpha = 0.05);

    environment save;
run;

/* Create cash flow data*/
data RDexamp.Inst_Data;
    informat _cfdate_ date7.;
    informat Mat_Date date7.;
    format Mat_Date date7.;
    format _cfdate_ date7.;
    length Currency $3.;
    length _cfcur_ $3.;
    input InstType $ InstID $ Par_LC Coup_Freq Coupon Mat_Date
        Company $ Currency $ _cfcur_ $ _cfdate_ _cfamnt_;
    datalines;
CF_Bond bond1 100000 6 0.05 01Jan03 EuroCorp EUR EUR 01Jan03 2500
CF_Bond bond1 100000 6 0.05 01Jan03 EuroCorp EUR EUR 01Jan03 100000
CF_Bond bond2 100000 6 0.05 01Jan04 BritCorp GBP GBP 01Jan03 2500
CF_Bond bond2 100000 6 0.05 01Jan04 BritCorp GBP GBP 01Jun03 2500
CF_Bond bond2 100000 6 0.05 01Jan04 BritCorp GBP GBP 01Jan04 2500
CF_Bond bond2 100000 6 0.05 01Jan04 BritCorp GBP GBP 01Jan04 100000
CF_Bond bond3 100000 6 0.05 01Jan05 AmerCorp USD USD 01Jan03 2500
CF_Bond bond3 100000 6 0.05 01Jan05 AmerCorp USD USD 01Jun03 2500
CF_Bond bond3 100000 6 0.05 01Jan05 AmerCorp USD USD 01Jan04 2500
CF_Bond bond3 100000 6 0.05 01Jan05 AmerCorp USD USD 01Jun04 2500
CF_Bond bond3 100000 6 0.05 01Jan05 AmerCorp USD USD 01Jan05 2500
CF_Bond bond3 100000 6 0.05 01Jan05 AmerCorp USD USD 01Jan05 100000
CF_Bond bond4 100000 3 0.088 01Jan05 SelfCorp USD USD 01Jan03 -2200
CF_Bond bond4 100000 3 0.088 01Jan05 SelfCorp USD USD 01Apr03 -2200
CF_Bond bond4 100000 3 0.088 01Jan05 SelfCorp USD USD 01Jul03 -2200
CF_Bond bond4 100000 3 0.088 01Jan05 SelfCorp USD USD 01Oct03 -2200
CF_Bond bond4 100000 3 0.088 01Jan05 SelfCorp USD USD 01Jan04 -2200
CF_Bond bond4 100000 3 0.088 01Jan05 SelfCorp USD USD 01Apr04 -2200
CF_Bond bond4 100000 3 0.088 01Jan05 SelfCorp USD USD 01Jul04 -2200
CF_Bond bond4 100000 3 0.088 01Jan05 SelfCorp USD USD 01Oct04 -2200
CF_Bond bond4 100000 3 0.088 01Jan05 SelfCorp USD USD 01Jan05 -2200
CF_Bond bond4 100000 3 0.088 01Jan05 SelfCorp USD USD 01Jan05 -100000
CF_Bond bond5 50000 3 0.08 01Jan05 SelfCorp USD USD 01Jan03 -1000
CF_Bond bond5 50000 3 0.08 01Jan05 SelfCorp USD USD 01Apr03 -1000
CF_Bond bond5 50000 3 0.08 01Jan05 SelfCorp USD USD 01Jul03 -1000
CF_Bond bond5 50000 3 0.08 01Jan05 SelfCorp USD USD 01Oct03 -1000
CF_Bond bond5 50000 3 0.08 01Jan05 SelfCorp USD USD 01Jan04 -1000
CF_Bond bond5 50000 3 0.08 01Jan05 SelfCorp USD USD 01Apr04 -1000
CF_Bond bond5 50000 3 0.08 01Jan05 SelfCorp USD USD 01Jul04 -1000
CF_Bond bond5 50000 3 0.08 01Jan05 SelfCorp USD USD 01Oct04 -1000
CF_Bond bond5 50000 3 0.08 01Jan05 SelfCorp USD USD 01Jan05 -1000
CF_Bond bond5 50000 3 0.08 01Jan05 SelfCorp USD USD 01Jan05 -50000
;

proc risk;
    /* Create a time grid and a cross-classification */
    environment open = RDexamp.example3;
    
    /* Create a time grid */
    timegrid Grid1 (6 month, 12 month, 18 month, 24 month);
    
    /* Create a cross-classification */
    crossclass cc2 (Company);
    
    /* Save the environment */
    environment save;
run;
proc risk;
    environment open = RDExamp.example3;
    /* Create a Coupon Bond Instrument */
    instrument CF_Bond
        label = "Cash Flow Bond"
        basetype = Bond_Root
        variables = (Par_LC,
        Coup_Freq, Coupon,
        Mat_Date, Company)
        valrecord=cashflows
        methods = (price CF_Bond_Prc);
    /* Register the Instrument Data */
    instdata CF_Data
        file = RDexamp.Inst_Data
        format = cash_flow
        variables = (InstType InstID Par_LC Coup_Freq Coupon
        Mat_Date Company Currency _cfcur_ _cfdate_
        _cfamnt_);
    environment save;
run;

proc risk;
    environment open = RDexamp.example3;
    /* Create Portfolio List for Bonds */
    sources CF_List CF_Data;
    /* Create Portfolio File for Bonds */
    read sources = CF_List out = CF_File;
    environment save;
run;

proc risk; 
  		     environment open = RDExamp.example3; 
   	     /* Create a traditional analysis */ 
    		 cashflow cfstat 
        		analysis = traditional 
        		buckettype = simple 
        		evaldate = "01NOV02"d;
/* Create a covariance simulation analysis */ 
          simulation covsim 
             method = covariance 
             data = Hist_Data 
             interval = month  
             seed = 12345  
             ndraw = 1500 
             horizon = (1); 
   
    /* Create a cash flow analysis based on COVSIM */ 
          cashflow cfcov 
             analysis = covsim 
             buckettype = simple 
             evaldate = "01NOV02"d;
/* Create an analysis project */ 
     project simproj  
        data = (Mkt_Data)  
        portfolio = CF_File 
        analysis = (cfstat cfcov) 
        numeraire = USD 
        crossclass = cc2 
        cfbucket = Grid1 (REL) 
        rundate = "01NOV02"d 
        out = CF_Sim; 
   
     setoptions allcashflow; 
   
     /* Run an analysis project */ 
     runproject simproj date = "01NOV02"d; 
   
     environment save; 
  run;